@@ -20,6 +20,7 @@ namespace Tomboy
 		#if ENABLE_DBUS
 		static RemoteControl remote_control;
 		#endif
+        static Gtk.IconTheme icon_theme = null;
 
 		public static void Main (string [] args)
 		{
@@ -38,6 +39,10 @@ namespace Tomboy
 
 			Initialize ("tomboy", "Tomboy", "tomboy", args);
 
+            // Add private icon dir to search path
+            icon_theme = Gtk.IconTheme.Default;
+            icon_theme.AppendSearchPath (System.IO.Path.Combine (Defines.DATADIR, "tomboy/icons"));
+
 //   PluginManager.CheckPluginUnloading = cmd_line.CheckPluginUnloading;
 
 			// Create the default note manager instance.
@@ -50,12 +55,6 @@ namespace Tomboy
 			RegisterRemoteControl (manager);
 
 			SetupGlobalActions ();
-
-			#if !ENABLE_DBUS
-			if (cmd_line.NeedsExecute) {
-				cmd_line.Execute ();
-			}
-			#endif
 			ActionManager am = Tomboy.ActionManager;
 
 			ApplicationAddin [] addins =
@@ -64,6 +63,12 @@ namespace Tomboy
 				addin.Initialize ();
 			}
 
+			#if !ENABLE_DBUS
+			if (cmd_line.NeedsExecute) {
+				cmd_line.Execute ();
+			}
+			#endif
+
 			if (cmd_line.UsePanelApplet) {
 				tray_icon_showing = true;
 				is_panel_applet = true;
